package edu.ucr.cnc.cas.support.duo;

/**
 * Spring bean for holding Duo configuration parameters passed to it in duoConfiguration.xml.
 *
 * An example bean configuration could be:
 *
 * <pre>
 * {@code
 * <bean id="duoConfiguration" class="edu.ucr.cnc.cas.duo.config.DuoConfiguration">
 *      <constructor-arg index="0" value="api-XXXXXXX.duosecurity.com"/>
 *      <constructor-arg index="1" value="XXXXXXXXXXXXXXXXXXXXXXX"/>
 *      <constructor-arg index="2" value="XXXXXXXXXXXXXXXXXXXXXXX"/>
 *      <constructor-arg index="3" value="XXXXXXXXXXXXXXXXXXXXXXX"/>
 * </bean>
 * }
 * </pre>
 *
 * @author Michael Kennedy <michael.kennedy@ucr.edu>
 * @version 1.0
 * @see <a href="https://www.duosecurity.com/docs/duoweb">Duo Web Developer Documentation</a>
 */
public class DuoConfiguration {
    private String apiHost;
    private String integrationKey;
    private String secretKey;
    private String applicationKey;

    /**
     * Constructor
     *
     * @param apiHost API Host provided in Duo console
     * @param integrationKey integration key provided in Duo console
     * @param secretKey secret key provided in Duo console
     * @param applicationKey application key generated by Duo customer and only known to Duo customer
     */
    public DuoConfiguration(String apiHost, String integrationKey, String secretKey, String applicationKey) {
        this.apiHost = apiHost;
        this.integrationKey = integrationKey;
        this.secretKey = secretKey;
        this.applicationKey = applicationKey;
    }

    public String getApiHost() {
        return apiHost;
    }

    public void setApiHost(String apiHost) {
        this.apiHost = apiHost;
    }

    public String getIntegrationKey() {
        return integrationKey;
    }

    public void setIntegrationKey(String integrationKey) {
        this.integrationKey = integrationKey;
    }

    public String getSecretKey() {
        return secretKey;
    }

    public void setSecretKey(String secretKey) {
        this.secretKey = secretKey;
    }

    public String getApplicationKey() {
        return applicationKey;
    }

    public void setApplicationKey(String applicationKey) {
        this.applicationKey = applicationKey;
    }
}
